function [x,y,timetaken]=LV_MCMCMC7(n0,ntrans_cat,tset,m,scale,shift,startparms,prior_mean,reportfreq,checkfreq,checkfilename,seed,seedn,reltol,abstol,beta,swapfreq,stiff)
%Metropolis-coupled MCMC estimation for rate and initial state probability parameters in LV model for coral data
%lognormal proposal on one parameter at a time (each equally
%likely to be changed)
%This version allows scale to be different for each parameter and has
%exponential priors on the initial state proportions
%priors are also allowed to have means other than zero
%Specify initial conditions for each chain
%Assumes chain 1 is cold chain
%Writes out parameters without back-transforming
%SYNTAX
%[x,y,timetaken]=LV_MCMCMC7(n0,ntrans_cat,tset,m,scale,shift,startparms,prior_mean,reportfreq,checkfreq,checkfilename,seed,seedn,reltol,abstol,nchain,beta,swapfreq,stiff)
%INPUTS
%n0 is column vector of initial state counts for each of s states
%ntrans_cat is horizontally concatenated matrices of transition counts (first time
%interval is leftmost s columns, within each  submatrix element i,j is
%count of transitions from j to i)
%tset is set of observation times (excluding zero)
%m is number of generations
%scale is the standard deviation of the proposal distribution (normal): either a scalar or
%a vector with one value for each parameter)
%shift is the mean of the proposal distribution: scalar or vector
%startparms is an array of starting parameters (one row for each chain).
%Each row has initial state probabilities, then off-diagonal rates going
%down then across
%prior_mean (optional) is mean for exponential prior on rate parameters
%(default 1e4).
%reportfreq (optional) is how often to display progress (default every 1000
%generations)
%checkfreq (optional) is how often to save results to a file (default every
%1000 generations)
%checkfilename (optional) is the name of the file to save to (default:
%LV_MCMC_#####.txt, where ##### is current date)
%seed and seedn (both optional) are seeds for rand and randn respectively
%(defaults: current time)
%reltol is relative tolerance for ODE solution accuracy (default 1e-3)
%abstol is absolute tolerance for ODE solution accuracy (default 1e-6)
%beta is vector of chain temperatures (default is calculated with
%deltaT=0.5). Should always make the first element in this vector be 1 if
%specifying it.
%swapfreq is how often we try chain swapping (default 10)
%stiff is 1 for stiff solver (ode15s), 0 (default) for standard solver
%(ode45s)
%OUTPUTS
%x is an array with one row per generation (only includes the last checkfreq generations, which is all we keep in memory), and one layer per chain.
%For each row and layer, the columns are:
%Generation number
%Log likelihood.
%Acceptance probability for that generation's proposal.
%transformed accepted initial state probabilities (to back transform,
%exponentiate and divide by sum)
%logs of the estimates of the elements in the A matrix (going down then
%across, excluding diagonals)
%Size of change in the parameter we proposed changing (on log scale)
%Index of the
%parameter that we changed in the proposal
%Whether or not we accepted the proposal
%Log likelihood of the proposed parameters
%Indicator of whether there was a NaN/complex problem with the log likelihood for the proposed
%parameters (1 if there was, 0 otherwise: negative log likelihood set to
%Inf if this happens)
%y is an array with one row per generation (only includes the last
%checkfreq generations). Columns are:
%Generation number
%Log likelihoods of all chains.
%Indices of two chains we proposed swapping (zero if we didn't do that this iteration)
%Whether or not we accepted the swap (-1 if no swap proposed)
%timetaken is clock time used
%Output files: 
%The main output file, named checkfilename, has the same format as output y
%(see below), with a header row
%We also write a separate output file for each chain, called
%checkfilename_chain#, which has the same format as the corresponding layer
%of output x
%USES
%LV_ufun_nocomplex.m

tzero=clock;
nstate=length(n0);%number of states in the model
nvar=nstate^2;%number of parameters to be estimated
nprop=nstate;%number of initial state probabilities
nchain=size(startparms,1);%number of chains to run

if nargin<8
    prior_mean = 1e4;%default prior mean for rate parameters
end;
if nargin<9
   reportfreq=1e3; 
end
if nargin<10
   checkfreq=1e3;
end
if nargin<11
   checkfilename=['LV_MCMC_',date,'.txt'];
end;   
if nargin<12
    seed=sum(100*clock);
end;
if nargin<13
    seedn=sum(100*clock);
end;
if nargin<14
    reltol=1e-3;
end;
if nargin<15
    abstol=1e-6;
end;
if nargin<16
    deltaT=0.5;
    beta=1./(1+deltaT*(0:(nchain-1)));%chain temperatures
end;
if nargin<17
    swapfreq=10;
end;
if nargin<18
    stiff=0;
end;

if isscalar(scale)
    scale=ones(1,nvar)*scale;
end;
if isscalar(shift)
    shift=ones(1,nvar)*shift;
end;

rand('twister',seed); %seed the pseudorandom number generators
randn('state',seedn);
x=zeros(checkfreq,nstate^2+8,nchain);%one layer per chain: don't need to store all generations in memory
y=zeros(checkfreq,nchain+4);%this stores the log likelihoods of all chains and information on swapping
sy=size(y,2);

chf=fopen(checkfilename,'w+');%open the main output file
estastr=[];
for i=1:nstate
    estastr=[estastr,'x0_',num2str(i),'\t'];
end;
for i=1:nstate
    for j=1:nstate
        if j~=i
            estastr=[estastr,'a_',num2str(i),num2str(j),'\t'];
        end;
    end;
end;
chainstr='l1\t';
for i=2:nchain
    chainstr=[chainstr,'l',num2str(i),'\t'];
end;

fprintf(chf,['step\t',chainstr,'\tjswap\tkswap\tacceptswap\n']);%write header row for main file

chfset=zeros(nchain,1);%array of file identifiers for separate chain output files
for i=1:nchain%open separate chain output files
    chfset(i)=fopen([checkfilename,'_chain',num2str(i)],'w+');
    fprintf(chfset(i),['step\tl',num2str(i),'\tgamma\t',estastr,'proposedchange\tk\taccept\tproposedl\tproblem\n']);%write header row for chain file
end;

fmt=[repmat('%.16g\t',1,size(x,2)),'\n'];%format string for chain file writing
fmty=[repmat('%.16g\t',1,size(y,2)),'\n'];%format string for main file writing
loglikb=zeros(1,nchain);
logpriorb=zeros(1,nchain);
estb=zeros(nchain,nvar);

startparms=log(startparms);%we want to work with parameters on log scale

for chain=1:nchain%initialize each chain
    lb=LV_ufun_nocomplex(est_rescale(startparms(chain,:)',nprop),n0,ntrans_cat,tset,reltol,abstol,stiff); %log likelihood for initial values
    lpb=logprior(startparms(chain,:),prior_mean); %log of the prior probability for initial values
    loglikb(chain)=-lb;
    logpriorb(chain)=lpb;
    estb(chain,:)=startparms(chain,:);
    x(1,:,chain)=[1 loglikb(chain),NaN,estb(chain,:),NaN,NaN,NaN,NaN,NaN];
    y(1,:)=[1 loglikb 0 0 -1];
end
disp(['chain temperatures ',num2str(beta)]);
disp(['rep 1, l=',num2str(loglikb)]);

c=2;%counter for checkpointing
g=1;%counter for chain swapping
acc=zeros(1,nchain);
for i=2:m%iterate for m generations
    for chain=1:nchain%iterate each chain
        k=ceil(nvar*rand);%change only one parameter
        esta=estb(chain,:);
        ch=scale(k)*randn+shift(k);%amount by which we change depends on scale and shift for that parameter
        if shift(k)~=0
            qr=normpdf(-ch,shift(k),scale(k))/normpdf(ch,shift(k),scale(k));%Ratio of proposal probabilities
        else
            qr=1;
        end;
        esta(k)=esta(k)+ch; %normal proposal on one transformed parameter
        [loglika,problem]=LV_ufun_nocomplex(est_rescale(esta',nprop),n0,ntrans_cat,tset,reltol,abstol,stiff); %log likelihood for proposed values
        loglika=-loglika;
        logpriora=logprior(esta,prior_mean); %log of prior probability for proposed values
        gamma=min(1,(exp(loglika+logpriora-loglikb(chain)-logpriorb(chain))*qr)^beta(chain));%acceptance probability for heat parameter beta
        if gamma>rand;%accept
             estb(chain,:)=esta;
             loglikb(chain)=loglika;
             logpriorb(chain)=logpriora;
             acc(chain)=1;
        else
            acc(chain)=0;
        end
        x(c,:,chain)=[i loglikb(chain),gamma,estb(chain,:),ch,k,acc(chain),loglika,problem];%write to row c, layer chain of x
    end
    y(c,:)=[i loglikb 0 0 -1];
    if g==swapfreq%try swapping chains
        cj=ceil(nchain*rand);%first chain
        ck=ceil(nchain*rand);%second chain
        R=min(1,exp((beta(cj)-beta(ck))*(loglikb(ck)+logpriorb(ck)-loglikb(cj)-logpriorb(cj))));%acceptance ratio for chain swap
        if R>rand%swap
            tempest=estb(cj,:);
            templ=loglikb(cj);
            tempp=logpriorb(cj);
            estb(cj,:)=estb(ck,:);
            loglikb(cj)=loglikb(ck);
            logpriorb(cj)=logpriorb(ck);
            estb(ck,:)=tempest;
            loglikb(ck)=templ;
            logpriorb(ck)=tempp;
            y(c,sy-2:sy)=[cj ck 1];
        else%don't swap
            y(c,sy-2:sy)=[cj ck 0];
        end;
        g=1;
    else%increment g
        g=g+1;
    end
    
    if c==checkfreq
        for chain=1:nchain
            fprintf(chfset(chain),fmt,(x(:,:,chain)'));%write to chain file using fprintf
        end;
        fprintf(chf,fmty,y');%write to main file
        c=1;%set c to 1
    else
        c=c+1;%increment c
    end
   
    if rem(i,reportfreq)==0%display progress
        disp(['rep ',num2str(i),', l=',num2str(loglikb)]);
    end

end
fclose(chf);%close main output file
for i=1:nchain
    fclose(chfset(i));%close chain output files
end;
timetaken=etime(clock,tzero);
end

%back-transform and rescale initial state probs to sum to 1
function estscale=est_rescale(est,nprop)
    est=exp(est);
    es=sum(est(1:nprop));
    estscale=est;
    estscale(1:nprop)=estscale(1:nprop)/es;
end

%log of prior probability for log-transformed parameters
function lp=logprior(parms,prior_mean)
    lp=sum(log(exppdf(exp(parms),prior_mean)));
end