function [negl,problem,Pset_cat]=LV_ufun_nocomplex(est,n0,ntrans_cat,tset,reltol,abstol,stiff)
%calculate negative log likelihood for an LVlike model, given initial state
%probabilities and interaction parameters
%SYNTAX
%[negl,problem]=LV_ufun_nocomplex(est,n0,ntrans_cat,tset,reltol,stiff)
%INPUTS
%est is parameters: first s are initial probs, remainder
%are aij, excluding diagonals, going down then across
%n0 are observed initial numbers in each state
%ntrans_cat is concatenated counts of transitions observed: s rows (where s is
%number of states), s*nt cols (where nt is number of time intervals): e.g.
%[ntrans{1} ntrans{2} ... ] from LVlike_sim.m
%tset is the set of time intervals at which observations were taken (first
%observation assumed to be at zero)
%reltol (optional, default 1e-3) is relative tolerance for ODE solution
%accuracy
%abstol (optional, default 1e-6) is absolute ODE solution tolerance
%stiff (optional, default 0) is 1 if we want a stiff solver, zero otherwise
%OUTPUTS
%negl is negative log likelihood
%problem is zero if there were no problems, or 1 if the log likelihood
%evaluated to NaN or complex (and was set to Inf as a result)
%Pset_cat is predicted transition probabilities (concatenated)
%USES
%LVlikefun.m
%ode_nstate_P_3.m
%NOTES
%all those calls to ode_nstate_P might be expensive.  Maybe try to rewrite
%so it gets all the P matrices in one call, if it's too slow?
%this version sets negl to Inf if there are numerical problems

if nargin<5
    reltol=1e-3;
end;
if nargin<6
    abstol=1e-6;
end;
if nargin<7
    stiff=0;
end;

s=length(n0);

p0=est(1:s);%initial state probs

A=zeros(s);
ind=find(~eye(s));
temp=est(s+1:length(est));

A(ind)=temp;%interaction coefficients, zeros on diagonal

%make into P matrices
t0=0;
Pset_cat=[];
xnow=p0;
for m=1:length(tset)
    tint=tset(m)-t0;%time interval for this P matrix
    [P,odet,x,x_P]=ode_nstate_P_3(tint,xnow,A,reltol,abstol,stiff);%solve the ODEs to get P matrix for each observation point
    t0=tset(m);
    lastt=length(odet);
    x=x(lastt,:);%the final values
    xnow=[x(1:s)]';%update the true probabilities after this step
    Pset_cat=[Pset_cat P];
end;
%get negative log likelihood

negl=-LVlikefun(n0,ntrans_cat,p0,Pset_cat);

if isnan(negl) || ~isreal(negl)
    disp(['problem: negative log likelihood=',num2str(negl),', setting to Inf'])
    negl=Inf;%shouldn't really happen but may avoid taking a bad step if an intermediate value is complex
    problem=1;
else
    problem=0;
end;
