function [P,t,x,x_P]=ode_nstate_P_3(tset,x0,A,reltol,abstol,stiff)
%get a P matrix for a multi-state L-V-like model
%this version forces the solution to be non-negative (which  we know it
%should be) and allows you to set the relative tolerance and choose a stiff
%solver)
%SYNTAX
%[P,t,x,x_P]=ode_nstate_P_3(tset,x0,A,reltol,abstol,stiff)
%INPUTS
%tset is time points at which we want info (in ascending order)
%x0 is initial conditions
%A is matrix of interaction rates: aij is rate from j to i
%(last state is assumed to be empty space, and diagonal elements must be
%zero)
%reltol is relative tolerance for solution accuracy (optional, default
%1e-3)
%abstol is absolute tolerance for solution accuracy (optional, default
%1e-6)
%stiff is 1 if we want a stiff solver, zero otherwise (optional, default zero)
%OUTPUTS
%P is the matrix whose entries p_ij are conditional probabilities of state
%i at final time
%given state j at time 0
%t is a vector of times
%x is corresponding values of:
%x1...x_n: state probabilities, where x_n is empty space
%p_11...p_n1: conditional probabilities of each state given we started in
%state 1
%...
%p_1,(n-1)...p_n,(n-1): conditional probabilities of each state given we
%started in state n-1
%all these found by Runge-Kutta
%(we don't directly get p_1,n...p_n,n by integration because they can be
%found from the others)
%x_P is the predicted proportions of each state found from making the pij
%into a P matrix, then x_P=P*x_0 (should match with x(length(t),1:nstate))
%USES
%ode_nstate_Pnest (a nested function) via ode45 or ode15s
%NOTES
%Only returns P at the final time interval, but does do x at the
%intermediate times as well

if nargin<4
    reltol=1e-3;
end;
if nargin<5
    abstol=1e-6;
end;
if nargin<6
    stiff=0;
end;
nstate=size(A,1);
temp=eye(nstate);
temp=temp(1:nstate,1:nstate-1);
init=[x0;reshape(temp,nstate*(nstate-1),1)];
options=odeset('RelTol',reltol,'NonNegative',1:nstate^2,'AbsTol',abstol);
if stiff
    [t,x]=ode15s(@ode_nstate_Pnest,[0 tset],init,options);%stiff solver
else
    [t,x]=ode45(@ode_nstate_Pnest,[0 tset],init,options);%standard solver
end;
    %nested function to do the integration%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    function dxdt=ode_nstate_Pnest(t,x)
        xm=reshape(x,nstate,nstate);%turn the vector back into a matrix
        %build the Q matrix
        xv=xm(:,1);
        xv(nstate)=1;
        X=diag(xv);
        xa=X*A;
        R=xa-diag(sum(xa,1));
        dxdt=R*xm;
        dxdt=dxdt(:);%must return a vector
    end%end nested function%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

r=length(t);

%build the P matrix
temp=[x(r,nstate+1:size(x,2)) zeros(1,nstate)]';
P=reshape(temp,nstate,nstate);

%work out the last row of the P matrix
psum=P*x0;
P(1:nstate,nstate)=(x(r,1:nstate)-psum(1:nstate)')/x0(nstate);
x_P=P*x0;
end
