function [endvals,nband,timetaken]=plotLV_MCMCMC7_posterior(estscale,istart,istep,istop,tsolve,tset,ntrans_cat,titles,bands,nbin,startyear,startoffset)
%plot sample from posterior of MCMCMC run (e.g. LV_MCMCMC7): trajectories
%and histograms of abundance at the end of the run
%SYNTAX
%[endvals,nband,timetaken]=plotLV_MCMCMC7_posterior(estscale,istart,istep,istop,tsolve,tset,ntrans_cat,titles,bands,startyear,startoffset)
%INPUTS
%estscale is output from MCMCMC_backtransform
%istart is start iteration from which we want to sample
%istep is interval between iterations
%istop is end iteration
%tsolve is max time at which we want ODE solution
%tset is the set of times at which observations were made
%ntrans_cat is observed counts of transition probabilities
%titles is names of the states
%bands (optional) is vector of cut points of three colour bands: colour trajectories by the final value of
%free space e.g. cut at 0.5, 0.75
%nbin (optional) is number of bins to use in histograms at ends of run
%(default 10)
%startyear (optional) is value to put at left-hand end of horizontal axis
%(default is zero). For example, if startyear=1963, the values on the axis
%will be 1963,1964,... instead of 0,1,...
%startoffset (optional) is number of years between 1 Jan in startyear and
%the date of the first sample (e.g. 0.51)
%OUTPUT
%a graph with one panel per state, observed counts plotted as circles, and
%samples from posterior projection of state abundances plotted as lines
%a histogram of abundance at the end of the run is plotted vertically on
%the right for each state
%endvals is values at time tsolve for each sampled trajectory (row) in each
%state (column)
%nband is number of trajectories in each band (if bands specified; zeros otherwise)
%timetaken is clock time used
%USES
%get_nsamp.m
%ode_nstate_P_3.m

if nargin<9
    bands=[];
end;
if nargin<10
    nbin=10;
end;
if nargin<11
    startyear=0;
end;
if nargin<12
    startoffset=0;
end;

tzero=clock;

figure(1);
clf;
nstate=size(ntrans_cat,1);
hset=zeros(nstate);%handles of trajectory axes
h2set=hset;%handles of histogram axes


[n0a,nsampa,p0a,psampa]=get_nsamp(ntrans_cat);
for n=1:nstate
    hset(n)=subplot(ceil(sqrt(nstate)),ceil(sqrt(nstate)),n);
    pos=get(hset(n),'position');
    w=pos(3);
    wide=0.9;
    pos(3)=w*wide;%make axis narrower than default
    set(hset(n),'position',pos);
    plot([0 tset]+startyear+startoffset,[p0a(n) psampa(n,:)],'ko');%plot the observed counts
    set(gca,'xlim',[0 tsolve+1+startoffset]+startyear);%want this axis to end close to last solution time
    hold on;
    pos2=pos;
    pos2(1)=pos(1)+w*wide;
    pos2(2)=pos2(2);
    pos2(3)=w*(1-wide);
    pos2(4)=pos(4);
    h2set(n)=subplot('position',pos2);%create new subplot next to the current one, on which we'll draw histogram
end;
ts=1:1:tsolve;%could go in smaller steps if we need smoother curves
nband=zeros(3,1);
endvals=zeros(length(istart:istep:istop),nstate);
c=1;
for i=istart:istep:istop
    %get sample from posterior
    x0=estscale(i,1:nstate);
    A=zeros(nstate);
    A(~eye(nstate))=estscale(i,nstate+1:nstate^2);
    %solve ODEs for these parameters
    [P,t,x]=ode_nstate_P_3(ts,x0',A,1e-3,1e-6,1);%use stiff solver
    endvals(c,:)=x(size(x,1),1:nstate);
    c=c+1;
    for n=1:nstate
        axes(hset(n));
        %add to plot
        if isempty(bands)
            plot([0 ts]+startyear+startoffset,x(:,n),'k-');
        else%pick colour by band
            if x(size(x,1),nstate)<bands(1)
                plot([0 ts]+startyear+startoffset,x(:,n),'k-');
                if n==1
                    nband(1)=nband(1)+1;
                end;
            elseif x(size(x,1),nstate)<bands(2)
                plot([0 ts]+startyear+startoffset,x(:,n),'b-');
                if n==1
                    nband(2)=nband(2)+1;
                end;
            else
                plot([0 ts]+startyear+startoffset,x(:,n),'r-');
                if n==1
                    nband(3)=nband(3)+1;
                end;
            end;
        end;
        xlabel('time (years)');
        ylabel(titles{n});
        axis auto;
        set(gca,'xlim',[0 tsolve+1]+startyear);%want this axis to end close to last solution time
    end;
end;

for n=1:nstate
    axes(h2set(n));
    [N,x]=hist(endvals(:,n),nbin);%plot histogram of densities at end
    barh(x,N,'hist');
    h=findobj(gca,'type','patch');
    set(h,'facecolor','none','edgecolor','k');%make the histograms unfilled
    xlabel('frequency');
    
%   kernel density estimate (seemed to be hard to get the right bandwidth)    
%    [f,xi,u]=ksdensity(endvals(:,n));
%    u=u/5;
%    [f,xi]=ksdensity(endvals(:,n),'width',u);
%    plot(f,xi,'k-');%plot kernel density at end of run
%    xlabel('density');

    yl=get(hset(n),'ylim');%set vertical axis limit same as main plot
    set(gca,'ylim',yl,'xaxislocation','top','yaxislocation','right');
    set(hset(n),'ylim',yl);
    set(gca,'yticklabel',[],'fontsize',8);
    xl=get(gca,'xlim');
    yl=get(gca,'ylim');
    text(xl(1)+0.2*(xl(2)-xl(1)),yl(2)-0.1*(yl(2)-yl(1)),char(96+n+nstate),'fontsize',10);%labels follow on from main plots
    axes(hset(n));
    xl=get(gca,'xlim');
    yl=get(gca,'ylim');
    text(xl(1)+0.1*(xl(2)-xl(1)),yl(2)-0.1*(yl(2)-yl(1)),char(96+n),'fontsize',10);%labels a,b,c...
end;
timetaken=etime(clock,tzero);