function plot_marginal_posteriors(chainjoin,burnin,thin,s,nbin,fontsize,parmchoice,vpos)
%plot marginal posterior distributions of parameters from LV_MCMCMC_7
%SYNTAX
%plot_marginal_posteriors(chainjoin,burnin,thin,s,nbin,fontsize,parmchoice)
%INPUTS
%chainjoin is cold chain output from LV_MCMCMC7 (possibly stitched together from
%several stages of run)
%burnin is number of iterations to discard as burnin
%thin is how often to sample the chain after burnin
%s is number of states
%nbin is number of bins to use in histograms
%fontsize is font size to use for tick labels
%parmchoice is the starting parameter values
%vpos (optional) is vertical position of panel label (between 0 and 1,
%default 0.7)
%OUTPUTS
%A set of s x s histograms. Initial state probabilities on diagonal (all set to [0 1] on horizontal axis),
%interaction coefficients off diagonal.
%USES
%MCMCMC_backtransform.m
%NOTES
%Will need to move the axis labels by hand

if nargin<8
    vpos=0.7;
end;

figure(1);
clf;
r=size(chainjoin,1);


estscale=MCMCMC_backtransform(chainjoin(:,4:s^2+3),s);
A=zeros(s);
A(logical(~eye(s)))=s+1:s^2;
ym=0;
hset=zeros(s^2);
c=1;
B=zeros(s);
B(logical(~eye(s)))=s+1:s^2;
ind=logical(eye(s));
B(ind)=1:s;
B=B';
porder=B(:);%order in which we get parameters out of parmchoice to show starting points
for i=1:s
    for j=1:s
        subplot(s,s,(i-1)*s+j);
        pletter=(i-1)*s+j;%alphabetical labelling of panels
        if pletter<=26
            pstr=char(96+pletter);
        else%aa, ab, ... if we have more than 26 panels
            pstr=[char(97) char(96+pletter-26)];
        end;

        if j~=i
            hist(estscale(burnin:thin:r,A(i,j)),nbin);%interaction rates on off-diagonals
            hset(c)=title([pstr,': \ita\rm_{',num2str(i),num2str(j),'}']);
            c=c+1;
        else
            n=histc(estscale(burnin:thin:r,i),0:1/nbin:1);%initial state proportions on diagonal
            bar(0:1/nbin:1,n,'histc');
            hset(c)=title([pstr,': \itp\rm_',num2str(i),'(0)']);            
            c=c+1;
            set(gca,'xlim',[0 1]);%initial state proportions all on natural [0 1] scale
        end;
        h=findobj(gca,'type','patch');
        set(h,'facecolor','none','edgecolor','k');%make the histograms unfilled
        yl=get(gca,'ylim');
        if yl(2)>ym
            ym=yl(2);
        end;
    end;
end;

for i=1:s^2
    h=subplot(s,s,i);
    row=floor((i-1)/s);
    col=rem(i-1,s);
    set(gca,'fontsize',fontsize);
    p=get(gca,'pos');
    set(h,'pos',[p(1)+0.025*col-0.09 p(2)-0.0095*row p(3)*1.15,p(4)*1.15]);%make panels a bit bigger
    set(gca,'ylim',[0 ym]);%same vertical scale on all
    
    %insert the vertical lines showing start points
    hold on
    for j=1:size(parmchoice,1)
        plot(parmchoice(j,porder(i))*ones(1,2),[0.9*ym 0.95*ym],'k-');
    end;
    
    if i==1
        [mant,expnt]=mantexpnt(str2num(get(gca,'yticklabel')));
        [m2,ex2]=mantexpnt(get(gca,'ylim'));
        expnt=max([expnt;ex2(2)]);%awkward: we need this if tick labels don't contain exponent
        set(gca,'yticklabel',mant);%only mantissa on left edge panels
        ylabel(['Frequency \times 10^{',num2str(max(expnt)),'}'],'fontsize',10);%y label contains exponent
    elseif rem(i,s)==1
        mant=mantexpnt(str2num(get(gca,'yticklabel')));
        set(gca,'yticklabel',mant);%only mantissa on left edge panels
    else        
        set(gca,'yticklabel',[]);%no tick labels on internal panels
    end;
    if i==s^2
        xlabel('Parameter value','fontsize',10);
    end
    xl=get(gca,'xlim');
    yl=get(gca,'ylim');
    set(gca,'xlim',xl,'ylim',yl);
    set(hset(i),'Position',[xl(1)+(xl(2)-xl(1))/2,vpos*yl(2)]);%move label to the right place
end;
end

function [mant,expnt] = mantexpnt(arg)
% MANTEXPNT
% Returns the mantissa and exponent of a real base 10 argument.
%taken from Matlab Central newsgroup and modified for vectors
% TEST DATA:
% arg = (0.5 - rand) * 10^fix(10*(0.5-rand))
% sprintf('\n\t%23.15E\n',arg)

    sgn = sign(arg);
    expnt = fix(log10(abs(arg)));
    mant = sgn .* 10.^(log10(abs(arg))-expnt);
    if abs(mant) < 1
        mant = mant * 10;
        expnt = expnt - 1;
    end
    mant(arg==0)=0; %quick fix, MS, 23/3/09
end